arch snes.cpu

//-------------------------------------------------------------------------------
// SRAM expansion 2.0
// by madsiur
// release date: 06/24/2024
// apply with bass
//-------------------------------------------------------------------------------
// slot 1 expanded to:  $306A00-$307FFF
//                      $306000-$3069FF = $1600-$1FFF
//
// slot 2 expanded to:  $316A00-$317FFF
//                      $316000-$3169FF = $1600-$1FFF
//
// slot 3 expanded to:  $326A00-$327FFF
//                      $326000-$3269FF = $1600-$1FFF
//
// other SRAM values:   $336000-$3375FF (game common SRAM)
//                      $337600-$337EFF (unused)
//                      $337F00 = last altered savefile
//                      $337F01 = rng index
//                      $337F08 = checksum value #$E41B (slot 1)
//                      $337F0A = checksum value #$E41B (slot 2)
//                      $337F0C = checksum value #$E41B (slot 3)
//                      $337F0E = checksum value #$E41B (slot 4)
//-------------------------------------------------------------------------------

macro seek(variable offset) {
  origin (offset & $3FFFFF)
  base offset
}

constant FREE_SPACE($C3F091)            // free space used by the hack
constant lbl_C37083($C37083)            // used for branching
constant lbl_C37047($C37047)            // used for branching

// internal header
seek($C0FFD8)
db $05                                  // SRAM size 5 = 256 Kbits

// rng index
seek($C3016F)
    lda $337F01                         // rng index in SRAM (formely $307FF1)
    inc                                 // rng index +1
    sta $337F01                         // save changes

// game saving
seek($C3151F)
    sta $337F00                         // set last altered file number (formley $307FF0)

// skins and skin palettes
seek($C31D36)
    lda $337F00                         // last altered file

seek($C33AAA)
    lda $30674E                         // skin slot 1 ($30674E)

seek($C33AC6)
    lda $31674E                         // skin slot 2 ($30714E)

seek($C33AE2)
    lda $32674E                         // skin slot 3 ($307B4E)

seek($C33AF9)
    lda $30674E                         // skin palette slot 1 ($30674E)

seek($C33B0D)
    lda $31674E                         // skin palette slot 2 ($30714E)

seek($C33B21)
    lda $32674E                         // skin palette slot 3 ($307B4E)

// validate SRAM markers
seek($C37028)
    cmp $337F08                         // in file a?
    beq lbl_C37047                      // okay if so
    cmp $337F0A                         // in file b?
    beq lbl_C37047                      // okay if so
    cmp $337F0C                         // in file c?
    beq lbl_C37047                      // okay if so
    cmp $337F0E                         // in file d?
    beq lbl_C37047                      // okay if so

// game saving
seek($C31540)
    jsr lbl_sram_save_game              // save game
    jmp lbl_C37083                      // mark all files

// game loading
seek($C3156A)
    jsr lbl_sram_load_game              // load game
    rts

// game loading
seek($C329EB)
    jsr lbl_sram_load_common            // load game common SRAM

// SRAM initialization
seek($C3704B)
    jsr lbl_sram_init_sram              // SRAM initialization
    rts

// file marking as
// Squaresoft property
seek($C37088)
    sta $337F08                         // mark file a (formely $307FF8)
    sta $337F0A                         // mark file b (formely $307FFA)
    sta $337F0C                         // mark file c (formely $307FFC)
    sta $337F0E                         // mark file d (formely $307FFE)

//=======================================
// free space code
//=======================================
//=======================================
// game saving
//=======================================
seek(FREE_SPACE)
scope lbl_sram_save_game: {
    clc 
    adc #$2F
    sta $E2
    rep #$20
    lda #$6000
    sta $E0
    tdc
    tax
    tay
smr_a:
    lda $1600,x
    sta [$E0],y
    inx
    inx
    iny
    iny
    cpx #$0A00
    bne smr_a
    tdc
    tax
srm_b:
    lda $336000,x
    sta [$E0],y
    inx
    inx
    iny
    iny
    cpx #$1600
    bne srm_b
    sep #$20
    rts
}

//=======================================
// game loading
//=======================================
scope lbl_sram_load_game: {
    clc
    adc #$2F
    sta $E2
    rep #$20
    lda #$6000
    sta $E0
    tdc
    tax
    tay
srm_a:
    lda [$E0],y
    sta $1600,x
    inx
    inx
    iny
    iny
    cpx #$0A00
    bne srm_a
    sep #$20
    rts
}

//=======================================
// game loading (common SRAM)
//=======================================
scope lbl_sram_load_common: {
    jsr $1566                           // load save file
    tdc
    lda $66
    clc
    adc #$2F
    sta $E2
    rep #$20
    lda #$6A00
    sta $E0
    tdc
    tax
    tay
load:
    lda [$E0],y
    sta $336000,x
    inx
    inx
    iny
    iny
    cpx #$1600
    bne load
    sep #$20
    rts
}
//=======================================
// SRAM init
//=======================================
scope lbl_sram_init_sram: {
    rep #$20
    tdc
    tax
init_sram:
    sta $306000,x                       // slot 1
    sta $316000,x                       // slot 2
    sta $326000,x                       // slot 3
    inx
    inx
    cpx #$2000
    bne init_sram
    sep #$20
    rts
}
